#include <avr/io.h>
#include <avr/sleep.h>
#include <avr/delay.h>
#include <avr/interrupt.h>
#include <uart.h>

//Puts pen on whiteboard or not, according to c
void set_pen(char c) {
    if (c) {
	PORTD|=(1<<5);
    } else {
	PORTD&=~(1<<5);
    }
}

//Get pen status
char get_pen(void) {
    return (PORTD&(1<<5))?1:0;
}

//Set motor m to position s
void set_motor(char m, char s) {
    char p,t;
    const char step[]={0x8,0xA,0x2,0x6,0x4,0x5,0x1,0x9};
    if (m==0) s>>=1; //motor 1 is half as fast as motor 0; we compensate for that here.
    p=step[s&0x7];
    t=PORTB;
    if (m==0) {
	t&=0xf0;
	t|=p;
    } else {
	t&=0x0f;
	t|=(p<<4);
    }
    PORTB=t;
}


char m1,m2,dir1,dir2,pen;
volatile int pos;
int steps,rm1,rm2;
unsigned short poweroff;

/*
Moving of the motors happen in the timer interrupt. One of the two motors
is moved at maximum speed, the other at a fraction of it. That happens by
putting the maximum value a motor has to move in the variable 'steps' and
iterating the variable 'pos' from 0 to steps. The motors then will get set to
amount_of_steps_for_that_motor*(pos/steps), which for one motor will be
one step per pos, for the other less.
*/

//Interrupt routine. Here, the moving of the motors happen.
ISR(TIMER1_COMPA_vect) {
    int p1,p2;
    //If we don't have to do something for a long time, withdraw the pen to save
    //current on the pen coil, and disable the stepper drivers to save power there.
    if (pos==steps || steps==0) {
	if (poweroff<2000) {
	    poweroff++;
	} else {
	    PORTB=0;
	    set_pen(0);
	}
	return;
    }
    poweroff=0;
    pos++;
    p1=((long)pos*(long)rm1)/steps;
    p2=((long)pos*(long)rm2)/steps;
    set_motor(0,(p1*dir1)+m1);
    set_motor(1,(p2*dir2)+m2);
    if (pos==steps) {
	//reset m1 and m2 to the current values, for the next move
	m1=((p1*dir1)+m1);
        m2=((p2*dir2)+m2);
    }
}

//This routine gets called by the rest of the code to set up a pen move.
//trm? is the amount of steps, dir? is the direction, pen indicates the
//pen is down.
void step_motors(int trm1,int trm2,char tdir1,char tdir2, char pen) {
    char waitfirst=0;
    char x;
    //On change pf direction, wait first.
//    if (pos!=steps) {
//	if ((dir1!=tdir1) || (dir2!=tdir2)) waitfirst=1;
//    }

    //If the pen gets withdrawn or extended, wait first.
    if (get_pen()!=pen) waitfirst=2;
    //If a previous move is active, wait till it's done.
    while (pos!=steps) sei();

    //Move pen and wait, if necessary
    set_pen(pen);
    if (waitfirst==1) {
	for (x=0; x<10; x++) _delay_ms(10);
    }
    if (waitfirst==2) {
	for (x=0; x<30; x++) _delay_ms(10);
    }

    //Set variables for the interrupt routine and return: the
    //interrupt routine will make sure the movement happens while the 
    //main routine can fetch other bytes from the uart.
    rm1=trm1; rm2=trm2; dir1=tdir1; dir2=tdir2;
    steps=0; pos=0;
    if (trm1>trm2) steps=trm1; else steps=trm2;
}

//Initialize steppers
void steppers_init() {
    pos=0; steps=0;
    DDRB=0xff;
    DDRD=0x20;
    //init timer1 to fire once every x msec
    TCCR1A=0;
    TCCR1B=0x8+3; // /64, reset on OCR1A
    OCR1A=150; //every x msec
    TIMSK|=0x40; //enable oc1a int
}


